/* public things from eval.c */

#ifndef _EVAL_H_
#define _EVAL_H_

#define STACK_OV_ERROR        1
#define STACK_UND_ERROR	      2
#define DYN_STACK_OV_ERROR    3
#define DYN_STACK_UND_ERROR   4
#define SYNTAX_ERROR          5
#define NO_OPERATOR_ERROR     6
#define NO_VALUE_ERROR        7
#define DIV_BY_ZERO_ERROR     8
#define OUT_RANGE_ERROR       9
#define MISSING_PAREN_ERROR  10
#define MISMATCH_PAREN_ERROR 11
#define INTERNAL_ERROR	     12
#define NOT_SUPPORTED_ERROR  13
#define NOT_DONE_ERROR       14
#define NO_MEM_ERROR         15
#define MEM_LIMIT_ERROR      16
#define MAX_LOOP_ERROR       17
#define NO_NUM_VALUE_ERROR   18
#define NO_STRING_ERROR      19
#define NO_LABEL_ERROR       20
#define MISSING_SEPARATOR_ERROR  21
#define HISTORY_RECURSION_ERROR  22
#define USER_BREAK		 23
#define OUT_OF_VAR_SPACE_ERROR   24
#define UNDEFINED_VARIABLE_ERROR 25
#define OUT_BASE_ERROR		 26
#define BAD_ATTR_ERROR		 27
#define INVALID_NAME_ERROR	 28

#define TYPE_NUM 1
#define TYPE_TXT 2
#define TYPE_NUM_VAR 3
#define TYPE_TXT_VAR 4

typedef struct {
  int  type;
  long num;      /* used for numeric types or as index for all variables */
  ptr  txt;      /* used for text types */
} object;

#define LEFT  1
#define RIGHT 2

#define BINARY     0
#define PRE_UNARY  LEFT
#define POST_UNARY RIGHT

#define MAX_OPERATOR 100
#define MAX_PRIORITY 127

#define LOWEST_UNARY_CODE 49
/*
 *  it would be 47, but operators 47 '(' and 48 ')'
 *  are treated separately
 */

enum op_codes {
  null=0, comma, eq, or_or_eq, xor_xor_eq, and_and_eq, or_eq, xor_eq, and_eq,
  lshift_eq, rshift_eq, plus_eq, minus_eq, times_eq, div_eq, ampersand_eq,
  or_or, xor_xor, and_and, or, xor, and,
  less, less_eq, greater, greater_eq, eq_eq, not_eq,
  lshift, rshift, plus, minus, times, division, ampersand,

  colon_less, colon_greater, less_colon, greater_colon,
  point_less, point_greater, less_point, greater_point,
  colon, point, question, another_null,

  left_paren, right_paren, not, tilde,
  pre_plus_plus, post_plus_plus, pre_minus_minus, post_minus_minus,
  star, print, _random_, _attr_, colon_question, point_question,
  pre_plus, pre_minus, a_circle, dollar, pre_null, post_null
};

typedef enum op_codes operator;

typedef struct {
    char priority, assoc, syntax, *name;
    operator code;
} operator_list;


typedef struct {
  object obj[MAX_STACK];
  int curr_obj;
  operator op[MAX_STACK];
  int curr_op;
} stack;

#define PRINT_NOTHING 0
#define PRINT_AS_PTR  1
#define PRINT_AS_LONG 2


int  eval_any	__P ((long *lres, ptr *pres, char **what));
int  evalp	__P ((            ptr *pres, char **what));
int  evall	__P ((long *lres,            char **what));
int  evaln	__P ((                       char **what));

void print_error __P ((int err_num));

extern char *error_msg[];
extern int error;
extern operator_list op_list[];

#define REAL_ERROR (error && error != USER_BREAK)
#define MEM_ERROR  (error == NO_MEM_ERROR || error == MEM_LIMIT_ERROR)

#endif /* _EVAL_H_ */

